﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ArcGIS.Core.CIM;
using ArcGIS.Core.Data;
using ArcGIS.Core.Geometry;
using ArcGIS.Desktop.Catalog;
using ArcGIS.Desktop.Core;
using ArcGIS.Desktop.Editing;
using ArcGIS.Desktop.Extensions;
using ArcGIS.Desktop.Framework;
using ArcGIS.Desktop.Framework.Contracts;
using ArcGIS.Desktop.Framework.Dialogs;
using ArcGIS.Desktop.Framework.Threading.Tasks;
using ArcGIS.Desktop.Mapping;

//Internal namespaces
using ArcGIS.Desktop.Internal.Editing;
using ArcGIS.Desktop.Internal.Mapping;

namespace ParcelFabricSDK
{
  internal class Merge : Button
  {
    protected async override void OnClick()
    {
      //jump to the cim thread
      await QueuedTask.Run(() =>
     {
        // check for selected layer
        if (MapView.Active.GetSelectedLayers().Count == 0)
       {
         System.Windows.MessageBox.Show("Please select a feature layer in the table of contents", "Merge");
         return;
       }

       string sTargetParcelType = Microsoft.VisualBasic.Interaction.InputBox("Target Parcel Type:", "Merge To Parcel Type", "Tax");

       if (sTargetParcelType.Trim().Length == 0)
         return;

        //first get the feature layer that's selected in the table of contents
        var featSrcLyr = MapView.Active.GetSelectedLayers().OfType<FeatureLayer>().First();
       var featTargetLyr = MapView.Active.Map.GetLayersAsFlattenedList().OfType<FeatureLayer>().FirstOrDefault(l => l.Name == sTargetParcelType);

       var myParcelFabricLayer = MapView.Active.Map.GetLayersAsFlattenedList().OfType<ParcelLayer>().FirstOrDefault();
       var pRec = myParcelFabricLayer.GetActiveRecord();

       if (pRec == null)
       {
         System.Windows.MessageBox.Show("There is no Active Record. Please set the active record and try again.", "Merge Parcels");
         return;
       }
       if (featSrcLyr.SelectionCount == 0)
       {
         System.Windows.MessageBox.Show("There is no selection on the source layer.", "Merge Parcels");
         return;
       }

       var opMerge = new EditOperation()
       {
         Name = "Merge",
         ProgressMessage = "Merging parcels...",
         ShowModalMessageAfterFailure = true,
         SelectNewFeatures = true,
         SelectModifiedFeatures = false
       };

       opMerge.Merge(featTargetLyr, featSrcLyr, featSrcLyr.GetSelection().GetObjectIDs());
       opMerge.Execute();
        //When active record is set, and merging into the same parcel type, the original parcels are set historic with RetiredByRecord GUID field
        //When active record is set, the new merged parcel will be tagged with the Active record.

      });
    }
  }
}
